package org.jboss.qa.guvnor;

import java.io.File;
import java.util.ArrayList;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.jboss.drools.guvnor.importgenerator.CmdArgsParser;
import org.jboss.drools.guvnor.importgenerator.ImportFileGenerator;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class MavenBulkImporterTest {
    // map of all the assets in package: Map<package_name, Map<asset_type, List<asset_name>>>
    private Map<String, Map<AssetType, List<String>>> assets;
    // list of all the categories in repository with all the predecessors
    private List<String[]> categories;
    // repository file in test_resources to test
    private static final String repository = "repository.xml";

    // types of assets
    private enum AssetType {
        BusinessRules, TechnicalRules, Function, DSL, Model, Process, Enumeration, TestScenario, XMLProperties, Other
    }

    public static void main(String[] arg) throws Exception {
        MavenBulkImporterTest test = new MavenBulkImporterTest();
        test.prepareXmlRepository();
    }

    public void loadRepositoryContent() throws Exception {
        assets = new HashMap<String, Map<AssetType, List<String>>>();

        File f = new File("test-resources", repository);
        Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(f);

        XPath xpath = XPathFactory.newInstance().newXPath();

        // get the package node
        Node n = (Node) xpath.evaluate("/node[1]/node[@name='drools:package_area']", doc, XPathConstants.NODE);

        NodeList nl = (NodeList) xpath.evaluate("node", n, XPathConstants.NODESET);

        // loop through packages
        Node pkg;
        for (int i = 0; i < nl.getLength(); i++) {
            pkg = nl.item(i);
            String pkgName = pkg.getAttributes().getNamedItem("sv:name")
                    .getTextContent();

            // TODO assets in globalArea
            if (pkgName.equals("globalArea")) {
                continue;
            }

            Map<AssetType, List<String>> pkgMap = assets.get(pkgName);

            if (pkgMap == null) {
                pkgMap = new EnumMap<AssetType, List<String>>(AssetType.class);
                assets.put(pkgName, pkgMap);
            }
            // get all assets
            NodeList content = (NodeList) xpath.evaluate(
                    "node[@name='assets']/node", pkg, XPathConstants.NODESET);

            Node asset;
            String assetName, assetType;
            List<String> assetList;
            for (int j = 0; j < content.getLength(); j++) {
                asset = content.item(j);
                assetName = asset.getAttributes().getNamedItem("sv:name")
                        .getTextContent();
                assetType = ((Node) xpath.evaluate(
                        "property[@name='drools:format']/value", asset,
                        XPathConstants.NODE)).getTextContent();

                assetList = pkgMap.get(getAssetType(assetType));

                if (asset.equals("drools") && assetType.equals("package")) {
                    //skip drools.package
                    System.err.println("I got drools.package");
                    continue;
                }

                if (assetList == null) {
                    assetList = new ArrayList<String>();
                    pkgMap.put(getAssetType(assetType), assetList);
                }

                assetList.add(assetName);
            }
            System.exit(0);
        }

        categories = new ArrayList<String[]>();

        // get the category node
        n = (Node) xpath.evaluate("/node[1]/node[@name='drools:tag_area']", doc, XPathConstants.NODE);

        loopCategories(n, new Stack<String>());

    }

    private AssetType getAssetType(String assetType) {
        if (assetType.equals("brl") || assetType.equals("xls")
                || assetType.equals("dslr")) {
            return AssetType.BusinessRules;
        }
        if (assetType.equals("drl")) {
            return AssetType.TechnicalRules;
        }
        if (assetType.equals("function")) {
            return AssetType.Function;
        }
        if (assetType.equals("dsl")) {
            return AssetType.DSL;
        }
        if (assetType.equals("jar") || assetType.equals("model.drl")) {
            return AssetType.Model;
        }
        if (assetType.equals("rf")) {
            return AssetType.Process;
        }
        if (assetType.equals("enumeration")) {
            return AssetType.Enumeration;
        }
        if (assetType.equals("scenario")) {
            return AssetType.TestScenario;
        }
        if (assetType.equals("xml") || assetType.equals("properties")) {
            return AssetType.XMLProperties;
        }

        return AssetType.Other;
    }

    private void loopCategories(Node start, Stack<String> previous)
            throws Exception {
        XPath xpath = XPathFactory.newInstance().newXPath();
        NodeList nl = (NodeList) xpath.evaluate("node", start,
                XPathConstants.NODESET);

        String name;
        Node n;
        for (int i = 0; i < nl.getLength(); i++) {
            n = nl.item(i);
            name = n.getAttributes().getNamedItem("sv:name").getTextContent();
            previous.push(name);

            categories.add(previous.toArray(new String[previous.size()]));
            loopCategories(n, previous);

            previous.pop();
        }
    }

    private void prepareXmlRepository() throws Exception {
        // don't do it by main method to get the exceptions
        File resources = new File("test-resources");
        File output = new File(resources, repository);
        if (output.exists()) {
            output.delete();
        }
        String[] args = new String[] {
                "-p", resources.getAbsolutePath(),
                "-s", ".",
                "-e", "1.0.0",
                "-r", "true",
                "-u", "tschloss",
                "-f", "drl,xls",
                "-o", output.getAbsolutePath(),
                "-b", new File(".").getAbsolutePath(),
                "-V"
        };

        CmdArgsParser parser = new CmdArgsParser();
        parser.parse(args);

        ImportFileGenerator ifg = new ImportFileGenerator();
        ifg.run(parser);
    }

}
